/**
 *   (c) 2001 Nathan Hjelm <hjelmn@unm.edu>
 *   v1.0.5 rio.h
 *
 *   header file for librioutil
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU Library Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 **/

#ifndef _RIO_H
#define _RIO_H

#include <sys/types.h>

#define s_int32_t signed long

/* errors */
#define URIO_SUCCESS 0
#define EOPEN        -32
#define EINTERFACE   -33
#define ECOMMAND     -34
#define EDELETE      -35
#define ERIORDY      -36
#define ENOINST      -37
#define ERIOFULL     -38
#define EREAD        -39
#define EWRITE       -40
#define ERIOOFF      -41
#define ENOFILE      -42

#define RIO600       1
#define RIO800       2
#define PSAPLAY      3
// note: rio one not yet supported
#define RIOONE       4
#define UNKNOWN      5

/*
  file types
*/
#define TYPE_MP3  0x3347504d
#define TYPE_WMA  0x20414d57
#define TYPE_WAV  0x504c4341
#define TYPE_WAVE 0x45564157

typedef struct _rio_mem {
  u_int32_t foo[4];
  u_int32_t size;
  u_int32_t used;
  u_int32_t free;
  u_int32_t system;
  
  u_int32_t foobar[8];
  
  u_int8_t name[64];
  
  u_int8_t unk[128];
} rio_mem_t;

/*
 * structure for a info header on the rio
 *
 */
typedef struct _rio_file {
  /* 0x0000 - 0x000f */
  u_int32_t file_no; /* set by rio ??? */
  /* 
     This appears to be a pointer to the start of the file.
     If it is left as 0x00000000 then the Rio seems to set it,
     but a value *can* be passed in with varying results.
  */
  u_int32_t start;


  u_int32_t size;
  u_int32_t time;

  
  /* 0x0010 - 0x001f */
  /* XXX time_t different sizes on different systems (64 bit on NetBSD/alpha) */
  u_int32_t mod_date; /* given in secs since epoch */

  /*
    So far I have found that this value contains this:
    0x00000001 0x00000010 0x00000100 must be set
    0x00000002 : has bookmark?
    0x00400000 : Name does not display
    0x00000080 : Downloadable
  */
  u_int32_t bits;

  /*
    This value can be set (currently) with the following values
    0x3347504d : MPG3 (MPEG Layer III)
    0x20414d57 : WMA  (Winbloze Media)
    0x504c4341 : ACLP (WAVE)
    0x45564157 : WAVE (Microshaft)
  */
  u_int32_t type; 

  u_int32_t foo3;

  
  /* 0x0020 - 0x002f */
  u_int32_t foo4; /* maybe channels? */
  u_int32_t sample_rate;
  u_int32_t bit_rate; /* it is multiplied by 128 so >> 7 gives value :) */
  u_int32_t foo5;
  
  /* 0x0030 - 0x005f -- usually zeros */
  u_int8_t foobar[48];
  
  /* 0x0060 - 0x007f */
  u_int8_t info0[32];
  
  /* 0x0080 - 0x00b9 */
  u_int8_t info1[64];
  
  /* 0x00c0 - 0x00ff */
  u_int8_t name[64];
  
  /* 0x0100 - 0x013f */
  u_int8_t title[64];
  
  /* 0x0140 - 0x017f */
  u_int8_t artist[64];
  
  /* 0x0180 - 0x01bf */
  u_int8_t album[64];
    
  /* 0x01c0 - ? */
  // TODO -- figure out what is here
  u_int32_t zeros[400];
} rio_file_t;

typedef struct _rio_prefs {
  /*
    0x00 - 0x0f Mostly known
  */
  u_int8_t unk0[4];
  u_int8_t eq_state;
  u_int8_t treble;
  u_int8_t bass;
  u_int8_t repeat_state;
  u_int8_t sleep_time;
  u_int8_t light_state;
  u_int8_t contrast;
  u_int8_t volume;
  u_int8_t unk1;
  u_int8_t unk2;
  u_int8_t unk3;
  u_int8_t playlist;

  u_int8_t unk4[48];

  u_int8_t name[16];

  /*
    i have only seen this set once
  */
  u_int8_t type[16];

  /*
    usually just 0's
  */
  u_int8_t unk5[1952];
} rio_prefs_t;

/*
  Playlist structure:

  first entry is always blank.
  name is copied from rio_file->name
  title is 32 characters from rio_file->title
  unk[3] is an integer whose value is the entry number
      (there are endianness issues to deal with here)

  Note:librioutil does not handle creation of playlists.
*/
typedef struct _rio_plst {
    struct song_list {
	u_int8_t  name[64];
	u_int32_t unk[8];
	u_int8_t  title[32];
    };

    struct song_list playlist[128];
} rio_plst_t;

/*
  Use these for list_flags in function return_list_rio
*/
#define RMP3 0x01
#define RWMA 0x02
#define RWAV 0x04
#define RDOW 0x08
#define RSYS 0x10
#define RLST 0x20
#define RALL 0x3f

typedef struct _file_list {
    rio_file_t *data;
    int num;

    /* file has a dummy header? */
    rio_file_t *dummy_data;
    int dummy_num;

    struct _file_list *prev;
    struct _file_list *next;
} file_list;

typedef struct _mem_list {
    rio_mem_t *data;
    file_list *files;

    u_int32_t total_time;
    u_int32_t num_files;
    
    u_int32_t last_file;
} mem_list;

typedef struct _rio_info {
    mem_list *memory;

    /*
      all of these can be changed and set :)
    */
    unsigned char name[16];
    u_int8_t  type;
    u_int8_t  light_state;
    u_int8_t  repeat_state;
    u_int8_t  eq_state;
    u_int8_t  bass;
    u_int8_t  treble;
    u_int8_t  sleep_time;
    u_int8_t  contrast;
    u_int8_t  playlist;
    u_int8_t  volume;

    /*
      Can not be manipulated.
    */
    /* this is most likely only 1 or 2 */
    u_int8_t total_memory_units;

    float version;
} rio_info_t;

typedef struct _rio_instance {
    /* void here to avoid the user needing to define WITH_USBDEVFS and such */
    void *dev;

    rio_info_t *info;

    int debug;
    int abort;

    unsigned char cmd_buffer[16];
    unsigned char buffer    [64];
    
    void (*progress)(int x, int X, void *ptr);
    void *progress_ptr;
} rio_instance_t;

typedef rio_instance_t *crioInstance;

/*
  rio funtions:
*/
rio_instance_t *init_instance_rio    (rio_instance_t *rio, int number, int debug);
void            delete_instance_rio  (rio_instance_t *rio);

s_int32_t       set_info_rio         (rio_instance_t *rio, rio_info_t *info);
s_int32_t       add_song_rio         (rio_instance_t *rio, u_int8_t memory_unit, char *file_name,
				      char *artist, char *title, char *album);
s_int32_t       delete_file_rio      (rio_instance_t *rio, u_int8_t memory_unit,
				      u_int32_t fileno);
s_int32_t       format_mem_rio       (rio_instance_t *rio, u_int8_t memory_unit);
s_int32_t       update_rio           (rio_instance_t *rio, char *file_name);
s_int32_t       download_file_rio    (rio_instance_t *rio, u_int8_t memory_unit, u_int32_t fileno,
				      char *fileName);
void            set_progress_rio     (rio_instance_t *rio, void (*f)(int x, int X, void *ptr), void *ptr);



/* new functions 8-8-2001 */
s_int32_t       update_info_rio      (rio_instance_t *rio);
rio_info_t     *return_info_rio      (rio_instance_t *rio);

/* new functions 9-13-2001 */
/* library info */
char          *return_conn_method_rio(void);

/*
  information on memory:

  memory_unit is an integer between 0 and MAX_MEM_UNITS - 1
*/
u_int8_t        return_mem_units_rio (rio_instance_t *rio);
u_int32_t       return_free_mem_rio  (rio_instance_t *rio, u_int8_t memory_unit);
u_int32_t       return_used_mem_rio  (rio_instance_t *rio, u_int8_t memory_unit);
u_int32_t       return_total_mem_rio (rio_instance_t *rio, u_int8_t memory_unit);
u_int32_t       return_num_files_rio (rio_instance_t *rio, u_int8_t memory_unit);
u_int32_t       return_time_rio      (rio_instance_t *rio, u_int8_t memory_unit);
file_list      *return_list_rio      (rio_instance_t *rio, u_int8_t memory_unit, u_int8_t list_flags);

char           *return_file_name_rio (rio_instance_t *rio, u_int32_t song_id, u_int8_t memory_unit);

/* XX  Future  XX */
u_int32_t       return_state_rio     (rio_instance_t *rio);
/* startup graphic seems to be changeable. this function will change it whenever it is figured out */
u_int32_t       set_gid_rio          (rio_instance_t *rio, char *file_name);

#endif /* _RIO_H */
